import click
import datetime
import json
import requests
from termcolor import colored

from .utils import load_config 

@click.command()
@click.argument('asset', type=str)
@click.option('--profile', default='prod', help='Profile to use')
def metrics(asset, profile):
    """List your asset's metrics"""
    # Get connector configs
    dagster_config = load_config('dagster', profile)

    # Set headers
    headers = {
        "Content-Type": "application/json",
        "Dagster-Cloud-Api-Token": dagster_config['api_token'],
    }

    asset_list = asset.split('/')

    query = f"""
    query AssetMetricsByKey {{
        assetOrError(assetKey: {{path: {json.dumps(asset_list)}}}) {{
            __typename
            ... on Asset {{
                id
                assetMaterializations(limit: 1){{
                    runId
                    timestamp
                    stepStats{{
                        stepKey
                        status
                        startTime
                        endTime
                    }}
                }}
                definition{{
                    freshnessInfo{{
                        currentMinutesLate
                    }}
                    partitionStats{{
                        numPartitions
                        numMaterialized
                        numFailed
                    }}
                }}
            }}
            ... on AssetNotFoundError {{
                message
            }}
        }}
    }}
    """

    response = requests.post(
        dagster_config['endpoint'], # type: ignore
        headers=headers, # type: ignore
        json={"query": query}
    )
    
    response.raise_for_status()
    
    data = response.json()

    if data["data"]["assetOrError"]["__typename"] == "AssetNotFoundError":
        click.echo(f"Error: {data['data']['assetOrError']['message']}")
    else:
        asset_info = data["data"]["assetOrError"]
    
        # Get AssetMaterializations attributes
        if asset_info['assetMaterializations']:
            first_materialization = asset_info['assetMaterializations'][0]
            run_id = first_materialization['runId'] if 'runId' in first_materialization else 'Not available'

            # Extract stepStats if available
            step_stats = first_materialization['stepStats'] if 'stepStats' in first_materialization else {}
            status = step_stats['status'] if 'status' in step_stats else 'Not available'
            
            if 'startTime' in step_stats:
                start_time = datetime.datetime.fromtimestamp(step_stats['startTime']).strftime('%Y-%m-%d %H:%M:%S')
            else:
                start_time = 'Not available'

            if 'endTime' in step_stats:
                end_time = datetime.datetime.fromtimestamp(step_stats['endTime']).strftime('%Y-%m-%d %H:%M:%S')
            else:
                end_time = 'Not available'

            # Calculate and format elapsed time
            if 'startTime' in step_stats and 'endTime' in step_stats:
                elapsed_seconds = step_stats['endTime'] - step_stats['startTime']
                elapsed_time = str(datetime.timedelta(seconds=elapsed_seconds))
            else:
                elapsed_time = 'Not available'
                
        else:
            run_id = 'Not available'
            status = 'Not available'
            start_time = 'Not available'
            end_time = 'Not available'
            elapsed_time = 'Not available'
        
        # Get Definition attributes
        if asset_info['definition']:
            definition = asset_info['definition']
            
            if 'partitionStats' in definition and definition['partitionStats'] is not None:
                partition_stats = definition['partitionStats']
                num_partitions = partition_stats['numPartitions'] if 'numPartitions' in partition_stats else 'Not available'
                num_materialized = partition_stats['numMaterialized'] if 'numMaterialized' in partition_stats else 'Not available'
                num_failed = partition_stats['numFailed'] if 'numFailed' in partition_stats else 'Not available'
            else:
                num_partitions = 'Not available'
                num_materialized = 'Not available'
                num_failed = 'Not available'
        else:
            num_partitions = 'Not available'
            num_materialized = 'Not available'
            num_failed = 'Not available'

        click.echo('\n')

        click.echo(colored("Latest Dagster materialization metrics:", 'magenta'))
        click.echo(colored(f"- Latest run ID: ", 'yellow') + colored(f"{run_id}", 'green'))
        click.echo(colored(f"- Status: ", 'yellow') + colored(f"{status}", 'green'))
        click.echo(colored(f"- Start time: ", 'yellow') + colored(f"{start_time}", 'green'))
        click.echo(colored(f"- End time: ", 'yellow') + colored(f"{end_time}", 'green'))
        click.echo(colored(f"- Elapsed time: ", 'yellow') + colored(f"{elapsed_time}", 'green'))

        click.echo('\n')

        click.echo(colored("Dagster partitions:", 'magenta'))
        click.echo(colored(f"- Number of partitions: ", 'yellow') + colored(f"{num_partitions}", 'green'))
        click.echo(colored(f"- Materialized partitions: ", 'yellow') + colored(f"{num_materialized}", 'green'))
        click.echo(colored(f"- Failed partitions: ", 'yellow') + colored(f"{num_failed}", 'green'))

        click.echo('\n')


import json
import click
import requests
from termcolor import colored

from .utils import load_config 

@click.command()
@click.argument('asset', required=True)
@click.option('--profile', default='prod', help='Profile to use')
def show(asset, profile):
    """Show details for a specific asset"""
    # Get connector configs
    dagster_config = load_config('dagster', profile)

    # Set headers
    headers = {
        "Content-Type": "application/json",
        "Dagster-Cloud-Api-Token": dagster_config['api_token'],
    }

    # Split the asset key into a list of strings
    asset_key = asset.split("/")

    # Insert asset_key directly into the query using f-string formatting
    query = f"""
    query AssetByKey {{
      assetOrError(assetKey: {{path: {json.dumps(asset_key)}}}) {{
        __typename
        ... on Asset {{
          definition {{
            description
            computeKind
            autoMaterializePolicy{{
              policyType
            }}
            freshnessPolicy{{
              maximumLagMinutes
              cronSchedule
            }}
            isPartitioned
            dependedByKeys{{
              path
            }}
            dependencyKeys{{
              path
            }}
          }}
          assetMaterializations(limit: 1){{
            timestamp
            metadataEntries{{
              __typename
              ... on FloatMetadataEntry{{
                label
                description
                floatValue
              }}
              ... on IntMetadataEntry{{
                label
                description
                intValue
              }}
              ... on JsonMetadataEntry{{
                label
                description
                jsonString
              }}
              ... on BoolMetadataEntry{{
                label
                description
                boolValue
              }}
              ... on MarkdownMetadataEntry{{
                label
                description
                mdStr
              }}
              ... on PathMetadataEntry{{
                label
                description
                path
              }}
              ... on NotebookMetadataEntry{{
                label
                description
                path
              }}
              ... on PythonArtifactMetadataEntry{{
                label
                description
                module
                name
              }}
              ... on TextMetadataEntry{{
                label
                description
                text
              }}
              ... on UrlMetadataEntry{{
                label
                description
                url
              }}
              ... on PipelineRunMetadataEntry{{
                label
                description
                runId
              }}
              ... on AssetMetadataEntry{{
                label
                description
                assetKey{{
                  path
                }}
              }}
              ... on NullMetadataEntry{{
                label
                description
              }}
            }}
          }}
        }}
        ... on AssetNotFoundError {{
          message
        }}
      }}
    }}
    """

    response = requests.post(
        dagster_config['endpoint'], # type: ignore
        headers=headers,
        json={"query": query}
    )
    
    response.raise_for_status()
    
    data = response.json()

    click.echo('\n')
    # Check if an error occurred and print the error message or asset details
    if data["data"]["assetOrError"]["__typename"] == "AssetNotFoundError":
        click.echo(f"Error: {data['data']['assetOrError']['message']}")
    else:
        asset_info = data["data"]["assetOrError"]
        # Use the get method to safely access the keys in the dictionary
        description = asset_info['definition'].get('description', 'Not available')
        compute_kind = asset_info['definition'].get('computeKind', 'Not available')
        is_partitioned = asset_info['definition'].get('isPartitioned', 'Not available')
        auto_materialize_policy_obj = asset_info['definition'].get('autoMaterializePolicy', None)
        auto_materialize_policy = auto_materialize_policy_obj.get('policyType', 'Not available') if auto_materialize_policy_obj is not None else 'Not available'

        # Check if freshnessPolicy is not None before accessing its fields
        freshness_policy = asset_info['definition'].get('freshnessPolicy', {})
        freshness_policy_lag = freshness_policy.get('maximumLagMinutes', 'Not available') if freshness_policy is not None else 'Not available'
        freshness_policy_cron = freshness_policy.get('cronSchedule', 'Not available') if freshness_policy is not None else 'Not available'

        click.echo(colored("Asset attributes:", 'magenta'))
        click.echo(colored(f"- Key: ", 'yellow') + colored(f"{asset}", 'green'))
        click.echo(colored(f"- Description: ", 'yellow') + colored(f"{description}", 'green'))
        click.echo(colored(f"- Compute kind: ", 'yellow') + colored(f"{compute_kind}", 'green'))
        click.echo(colored(f"- Is partitioned: ", 'yellow') + colored(f"{is_partitioned}", 'green'))
        click.echo(colored(f"- Auto-materialization policy: ", 'yellow') + colored(f"{auto_materialize_policy}", 'green'))
        click.echo(colored(f"- Freshess policy (maximum lag minutes): ", 'yellow') + colored(f"{freshness_policy_lag}", 'green'))
        click.echo(colored(f"- Freshess policy (cron schedule): ", 'yellow') + colored(f"{freshness_policy_cron}", 'green'))

        click.echo(colored("\nUpstream assets:", 'magenta'))
        upstream_assets = asset_info['definition']['dependencyKeys']
        if upstream_assets:
            for path in upstream_assets:
                click.echo(colored(f"- {'/'.join(path['path'])}", 'cyan'))
        else:
            click.echo("- None")

        click.echo(colored("\nDownstream assets:", 'magenta'))
        downstream_assets = asset_info['definition']['dependedByKeys']
        if downstream_assets:
            for path in downstream_assets:
                click.echo(colored(f"- {'/'.join(path['path'])}", 'cyan'))
        else:
            click.echo("- None")
        
        # Handle 'assetMaterializations'
        asset_materializations = asset_info.get('assetMaterializations', [])
        if asset_materializations:
            # Get the most recent materialization (it should be the first since we limited it to 1)
            last_materialization = asset_materializations[0]

            click.echo(colored("\nLatest materialization's metadata entries:", 'magenta'))
            timestamp = last_materialization.get('timestamp', 'Not available')
            click.echo(colored(f"- Last materialization timestamp: ", 'yellow') + colored(f"{timestamp}", 'green'))
            
            # Handle 'metadataEntries'
            metadata_entries = last_materialization.get('metadataEntries', [])
            if metadata_entries:
                for entry in metadata_entries:
                    label = entry.get('label', 'Not available')
                    description = entry.get('description', 'Not available')
                    typename = entry.get('__typename')

                    value = 'Not available'
                    if typename == 'FloatMetadataEntry':
                        value = entry.get('floatValue', 'Not available')
                    elif typename == 'IntMetadataEntry':
                        value = entry.get('intValue', 'Not available')
                    elif typename == 'JsonMetadataEntry':
                        value = entry.get('jsonString', 'Not available')
                    elif typename == 'BoolMetadataEntry':
                        value = entry.get('boolValue', 'Not available')
                    elif typename == 'MarkdownMetadataEntry':
                        value = entry.get('mdStr', 'Not available')
                    elif typename == 'PathMetadataEntry' or typename == 'NotebookMetadataEntry':
                        value = entry.get('path', 'Not available')
                    elif typename == 'PythonArtifactMetadataEntry':
                        value = f"module: {entry.get('module', 'Not available')}, name: {entry.get('name', 'Not available')}"
                    elif typename == 'TextMetadataEntry':
                        value = entry.get('text', 'Not available')
                    elif typename == 'UrlMetadataEntry':
                        value = entry.get('url', 'Not available')
                    elif typename == 'PipelineRunMetadataEntry':
                        value = entry.get('runId', 'Not available')
                    elif typename == 'AssetMetadataEntry':
                        asset_key_path = entry.get('assetKey', {}).get('path', 'Not available')
                        value = '/'.join(asset_key_path) if asset_key_path != 'Not available' else 'Not available'
                    elif typename == 'NullMetadataEntry':
                        value = 'Null'

                    click.echo(colored(f"- {label}: ", 'yellow') + colored(f"{value}", 'green'))
            else:
                click.echo("No metadata entries.")
        else:
            click.echo("No asset materializations.")
    click.echo('\n')


import json
import click
import requests
from termcolor import colored

from .utils import load_config 

@click.command()
@click.option('--prefix', default=None, help='Get list of assets with a given prefix')
@click.option('--profile', default='prod', help='Profile to use')
def ls(prefix, profile):
    """List your platform's data assets"""
    # Get connector configs
    dagster_config = load_config('dagster', profile)

    # Set headers
    headers = {
        "Content-Type": "application/json",
        "Dagster-Cloud-Api-Token": dagster_config['api_token'],
    }

    if prefix:
      prefix_list = prefix.split('/')
      query = f"""
      query AssetsQuery {{
        assetsOrError(prefix: {json.dumps(prefix_list)}) {{
          ... on AssetConnection {{
            nodes {{
              key {{
                path
              }}
            }}
          }}
        }}
      }}
      """
    else:
      query = """
      query AssetsQuery {
        assetsOrError {
          ... on AssetConnection {
            nodes {
              key {
                path
              }
            }
          }
        }
      }
      """

    response = requests.post(
        dagster_config['endpoint'], # type: ignore
        headers=headers, # type: ignore
        json={"query": query}
    )
    
    response.raise_for_status()
    
    data = response.json()

    # Extract asset keys and print them
    click.echo('\n')
    for node in data['data']['assetsOrError']['nodes']:
        asset_key = "/".join(node['key']['path'])
        click.echo(colored(f'- {asset_key}', 'cyan'))
    click.echo('\n')


import yaml
import os

def load_config(connector, profile):
    with open(os.path.expanduser('~/.damn/connectors.yml'), 'r') as file:
        config = yaml.safe_load(file)
    try:
        return config[connector][profile]
    except KeyError:
        raise ValueError(f"No configuration found for connector '{connector}' with profile '{profile}'")

